 
/*
Copyright (C) 2005, Scott Haywood, scotthaywood.com 
 
This is free software; you can redistribute it and/or modify 
it under the terms of the GNU General Public License as published by 
the Free Software Foundation; either version 2 of the License, or 
(at your option) any later version. 
 
This code is presented in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
GNU General Public License for more details. 
*/

/* 
 * Status:  This currently cannot handle large amounts of data;
 *          the Javascript interpreter gets bogged down.
 */

/* Initialization
 */
function myInit() { initMessage(); }

function initMessage() { setMessage("This space is used for messages"); }
 
/* Remove leading and trailing whitespace
*/
function stripWhiteSpace(str) {
	str = str.replace(/^\s+/,"");
	str = str.replace(/\s+$/,"");
	/* alert("str=|"+str+"|"); */
	return str;
}

/* Convert newline-separated strings into an array.
*  Strip out comments and unneeded spaces.
*/
function getRows(astring) {
	var aaa = new Array(); 
	var delim = "\n"; 
	var start = 0;
	var foundi = -1;
	var ii = 0;
	while (start < astring.length) {
		var tmp = astring.substring(start);
		foundi = tmp.search(delim);
		if (foundi == -1) {
			foundi = tmp.length;
		}
		/* alert("ii=" + ii +
			", start=" + start + ", foundi=" + foundi); */
		var xx = tmp.substring(0,foundi); /* excludes char at foundi */
		xx = stripWhiteSpace(xx);
		if (xx != "" && xx.substring(0,1) != '#') {
			aaa[ii] = xx;
			ii++;
		}
		start += foundi + 1;
	}
	return aaa; 
}
 
/* Convert newline-separated strings into an array.
*/
function getRows2(astring, skip) {
	//alert("getRows2: astring=" + astring);
	var aaa = new Array(); 
	var delim = "\n"; 
	var start = 0;
	var foundi = -1;
	var ii = 0;
	while (start < astring.length) {
		var tmp = astring.substring(start);
		foundi = tmp.search(delim);
		if (foundi == -1) {
			foundi = tmp.length;
		}
		//alert("getRows2: ii=" + ii +
		//	", start=" + start + ", foundi=" + foundi);
		var xx = tmp.substring(0,foundi); /* excludes char at foundi */
		if (1) {
			if (skip>0) {
				skip--;
			} else {
				aaa[ii] = xx;
				ii++;
			}
		}
		else { 
		 xx = stripWhiteSpace(xx);
		 if (xx != "" && xx.substring(0,1) != '#') {
			aaa[ii] = xx;
			ii++;
		 }
		}
		start += foundi + 1;
	}
	return aaa; 
}

/* When a waypoint is displayed, the coordinates are rounded
*  to this many places after the decimal point (trailing zeros are omitted).
*  More decimal places are shown if the input coordinates have more digits
*  after the decimal point.
*/
var orig_latlong_decimals = 6;
var latlong_decimals = 6;

// Return an HTML newline
function nl() { 
	return "<br>\n"; 
} 

/* Return a string to represent a waypoint
*/
function pointString(latitude, longitude) {
	var la = roundNumber(latitude, latlong_decimals);
	var lo = roundNumber(longitude, latlong_decimals);
	return "("+la+","+lo+")";
}

/* Return true if the given thing is a number.
*  Assume it's either a number or a string.
*  Note: this must work for 0 and "0"
*/
function isNumber(thing) {
	return "" + (thing + 1) != (thing + "" + 1);
}

// Return true if the given string looks like a number
//
function matchesNumber(thing) {
	return thing.match(/^-?\d*(\.\d+)?$/) ;
}


function bad_k1() {
	setMessage("column number '" + document.xyz.k1.value + "' is not valid", 1);
}	


/* Parse the input, compute and display results.
*  This is a top-level function.
*/
function createNewTable(a_flag) {
	
	//alert("document.all="+document.all);
	//alert("document.layers="+document.layers);
	//

	setMessage("");

	var k1=document.xyz.k1.value; 
	var x1=document.xyz.x1.value; 
	var y1=document.xyz.y1.value; 

	var k2=document.xyz.k2.value; 
	var x2=document.xyz.x2.value; 
	var y2=document.xyz.y2.value; 

	var fff=document.xyz.field1.value; 
	var ggg=document.xyz.field2.value; 

	var delim1=document.xyz.delim1.value; 
	var delim2=document.xyz.delim2.value; 

	var skip1=document.xyz.skip1.value; 
	var skip2=document.xyz.skip2.value; 

	var match1=document.xyz.match1.value; 

	if (delim1 == "\\t") {
		delim1 = "\t";
	}
	if (delim2 == "\\t") {
		delim2 = "\t";
	}
	if (k1 == "" || k1 < 1) { bad_k1(); return }


	//alert("fff='" + fff + "'");
	
	var aaa = getRows2(fff, skip1);

	var flag = getWaypointTable(aaa, match1, delim1, k1, x1, y1);
	if (flag == -1) {
		setMessage("bad waypoint table",1);
		document.xyz.field1.focus();
		return -1;
	}

	var bbb = getRows2(ggg, 0);
	var nskip2 = skip2;
	var nchanged = 0;
	var ccc = new Array();
	var got_data_err = 0;

	// this is too slow if lots of data
	//
	if (bbb.length > 100) {
		setMessage("there is a limit of 100 lines of target data.");
		return;
	}

	for (var i=0; i<bbb.length; i++) {

		if (i%10 == 1) {
			appendMessage("Completed " + Number(i-1).toString(10) + " lines!");
		}
		var row = bbb[i];
		if (nskip2 > 0) {
			ccc.push(row);
			nskip2--;
			continue;
		}
		var fields = row.split(delim2);
		var lim = fields.length + 1;
		if (lim > k2 && lim > x2 && lim > y2) {
			var key = fields[k2-1];
			//alert("got key " + key);
			if (key != "") {
				var wp = lookupWaypoint(key);
				//alert("lookup returns " + wp.length);
				if (typeof(wp) != 'undefined' && wp.length == 2) {
					fields[x2-1] = wp[0];
					fields[y2-1] = wp[1];
					ccc.push(fields.join(delim2));
					nchanged++;
				} else {
				 got_data_err++;
				 appendMessage("line " + i + ": waypoint '" + fields[k2-1] + "' not found"
						 + " in column " + k1 + " of waypoint table");
				}
			}
		}
	}
	//alert("created new table, size=" + ccc.length);

	appendMessage("waypoint table: " + getWaypointTableSize() + " entries");
	if (!got_data_err) appendMessage("target data: " + ccc.length + " rows, changed " + nchanged);
	highlightMessage();
	//alert("got_data_err = " + got_data_err + ", ccc.length = " + ccc.length);
	if ( got_data_err==0 && ccc.length > 0 ) {
		displayNewTable(ccc);
	}

	return 1;
}

function displayNewTable(rows) {
	var doc = "<pre>";
	doc += rows.join("\n");
	doc += "\n";
	doc += "</pre>";
	displayIt(doc, 0);
}

var wp_table = new Array();

function getWaypointTableSize() {
	return wp_table.length;
}

function getWaypointTable(rows, match, delim, k, x, y) {

 	var aaa = rows;
	wp_table = new Array();

	/**
	if (0) {
		var msg = "<pre># rows = " + aaa.length + nl();
		for (var i=0; i<aaa.length; i++) {
			msg += ":" + aaa[i] + ":\n";
		}
		msg += "</pre>";
		displayIt(msg,1);
	}
	**/

	if (match) {
		appendMessage("Using waypoint entries that contain '" + match + "'.");
	}

	var table_i = 0;
	var got_err = 0;
	//alert("getWaypointTable: got " + rows.length + " rows");
	for (var i=0; i<aaa.length; i++) {

		var row = aaa[i];
		//alert("getWaypointTable: row= '" + row + "'");

		if (match && ! row.match(match)) 
			continue;

		/* split the row, attempt using String.split */
		var fields = row.split(delim);
		//alert("getWaypointTable: " + fields.length + " fields");
		if (fields.length > 0) {

			// if a waypoint is seen
			// note: the indices k, x, y start at 1, not 0
			var lim = fields.length + 1;
			if (lim > k && lim > x && lim > y) {
			 wp_table[fields[k-1]] = new Array(fields[x-1],fields[y-1]);
			 //appendMessage("wp_table: key " + fields[k-1] + ",entry " + wp_table[fields[k-1]]);
			}
		}
	}
	//alert("waypoint table size: " + wp_table.length);
	if (got_err) {
		return -1;
	} else {
		return 0;
	}
}

function lookupWaypoint(key) {
	//alert("lookup: key=" + key);
	//alert("lookup: key=" + key + ", table=" + wp_table.join(","));
	var entry = wp_table[key];
	//if (typeof(entry) != 'undefined') { }
	return entry;
}


/* Highlight the message area.
*  This provides a visual confirmation that the 
*  computation has been performed (again), even when the 
*  message remains the same.
*/
var hm_color_index = 0;
var hm_colors = new Array("#990000", "#009900", "#000099");
var hm_strings = new Array("*", "#", "+"); // same array length
function highlightMessage() {
	var hci = hm_color_index + 1;
	if (hci >= hm_colors.length)
		hci = 0;
	hm_color_index = hci;
	var color = hm_colors[hci];

	var obj = document.xyz.message;
	if (obj.style) obj.style.color = color;
	else {
		var sav = document.xyz.message.value;
		var t= "************************";
		t = t.replace(/./g,hm_strings[hci].substring(0,1));
		document.xyz.message.value = t + sav;
	}

}

/* Set the text of the message display area.
*/
function setMessage(msg, hilite) {
	document.xyz.message.value = msg;
	if (hilite) document.xyz.message.style.backgroundColor = "yellow"
	else document.xyz.message.style.backgroundColor = "white";
}

/* Append text to the message display area.
*/
function appendMessage(msg) {
	var m = document.xyz.message.value;
	if (m != "")
		m += "\n";
	document.xyz.message.value = m + msg;
}

/* Create an HTML anchor to close a window
*/
function getCloseAnchor() { 
	var rv = String( 
		"<a href=\"javascript:window.close();\">" 
		+ "[ close ]</a>" 
		); 
	return rv; 
} 

/* Display some HTML in a new window
*/
function displayIt(doc, addClose) {
 
 var temp = "(html)(head)(title)Insert waypoints - results(/title)(/head)(body)\n" 
            +"(!-- courtesy of scotthaywood.com --)\n" 
			+"--doc--(/body)(/html)"; 
 temp = temp.replace(/\(/g, "<"); 
 temp = temp.replace(/\)/g, ">"); 
 temp = temp.replace(/--doc--/, doc + nl() + (addClose?getCloseAnchor():"")); 
 
 
 if (win2 != null) {
 	if (win2.close)
		win2.close();
	win2 = null;
 }
 win2 = window.open("", "results", "scrollbars=yes,menubar=yes,resizable=yes,toolbar=yes"); 
 win2.document.write(temp); 
 win2.document.close(); 
 win2.focus(); 
} 
var win2 = null; 


/* Html space
*/
function sp() {
	return '&nbsp;';
}

/* Html spaces
*/
function spn(n) {
	var s="";
	for (var i=0;i<n;i++) s += sp();
	return s;
}


/* Clear the input form and the message area.
*/
function clearForm() {
	document.xyz.field1.value = ""; 
	document.xyz.field2.value = ""; 

	document.xyz.delim1.value = "";
	document.xyz.delim2.value = "";

	document.xyz.x1.value = "";
	document.xyz.x2.value = "";

	document.xyz.y1.value = "";
	document.xyz.y2.value = "";

	document.xyz.k1.value = "";
	document.xyz.k2.value = "";

	document.xyz.skip1.value = "";
	document.xyz.skip2.value = "";

	document.xyz.match1.value = "";
	//document.xyz.match2.value = "";

	initMessage();

} 

function clearData() {
	document.xyz.field2.value = ""; 
}

// Examples of input data
//
var e_index = -1;
var examples = new Array(
	new Array(
		1,2,3,",",0,'',
		"wp1,45.12345,-111.12340\n"
		+"wp2,45.12346,-111.12341\n"
		+"wp3,45.12347,-111.12342\n"
		+"wp4,45.12348,-111.12343\n",
		1,3,4,"\\t",0,
		"wp2\tThing 2\txxx\tyyy\n"
		+"wp3\tThing 3\txxx\tyyy\n"
		+"wp4\tThing 4\txxx\tyyy\n"
		+"wp1\tThing 1\txxx\tyyy\n",
		"In this example, the waypoint table uses commas to separate the fields, "
		+"and the target data uses tabs"
		),
	new Array(
		2,3,4,",",1,'WP',
		"# waypoints entries begin with WP\n"
		+"WP,nest-C,45.12345,-111.12340\n"
		+"WP,nest-B,45.12346,-111.12341\n"
		+"WP,nest-A,45.12347,-111.12342\n"
		+"G,nest-A,22.0,3.3, this line not used",
		4,2,3,",",2,
		"# leave this line unchanged\n"
		+"# and this line, too\n"
		+"this is nest A,,,nest-A\n"
		+"also nest A,,,nest-A\n"
		+"second,,,nest-B\n"
		+"third,,,nest-C\n",
		"Note the use of a pattern, \"WP\", to select the waypoint entries."
		),
	new Array(
		1,5,6,"\\t",0,'alt=',
		"This line is ignored\n"
		+"HPG5\tcomment\t12\tT\t305021\t5023455\talt=1801\n"
		+"HPG6\tcomment\t12\tT\t305340\t5023466\talt=1702\n"
		+"HPG7\tcomment\t12\tT\t305345\t5023477\talt=1503\n"
		+"this line is ignored",
		24,25,26,"\\t",1,
		"# leave this line unchanged\n"
		+"a\tb\tc\td\te\tf\tg\th\ti\tj\tk\tl\tm\tn\to\tp\tq\tr\ts\tt\tu\tv\tw\tHPG6\txx\tyy\tjunk\n"
		+"a\tb\tc\td\te\tf\tg\th\ti\tj\tk\tl\tm\tn\to\tp\tq\tr\ts\tt\tu\tv\tw\tHPG7\txx\tyy\tjunk\n",
		"In this example, the waypoint lines containing \"alt=\" are used."
		+"  The UTM coordinates will be copied into the long lines of the target data."
		)
	);

// Insert example(s) into the input area.
//
function insertExample() {
	var i = e_index + 1;
	if (i >= examples.length)
		i = 0;
	e_index = i;
	var xxx = examples[i];
	if (xxx.length < 10) {
		alert("bad example, # " + i);
	}
	document.xyz.k1.value = xxx[0];
	document.xyz.x1.value = xxx[1];
	document.xyz.y1.value = xxx[2];
	document.xyz.delim1.value = xxx[3];
	document.xyz.skip1.value = xxx[4];
	document.xyz.match1.value = xxx[5];
	document.xyz.field1.value = xxx[6];
	document.xyz.k2.value = xxx[7];
	document.xyz.x2.value = xxx[8];
	document.xyz.y2.value = xxx[9];
	document.xyz.delim2.value = xxx[10];
	document.xyz.skip2.value = xxx[11];
	document.xyz.field2.value = xxx[12];
	setMessage(xxx[13]);

} 
 
